@@ -1,10 +1,10 @@
 # -*- perl -*-
 
 #
-# $Id: Date.pm,v 1.57 2005/08/25 20:39:30 eserte Exp $
+# $Id: Date.pm,v 1.70 2008/09/23 19:28:59 eserte Exp $
 # Author: Slaven Rezic
 #
-# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2005 Slaven Rezic. All rights reserved.
+# Copyright (C) 1997, 1998, 1999, 2000, 2001, 2005, 2007, 2008 Slaven Rezic. All rights reserved.
 # This package is free software; you can redistribute it and/or
 # modify it under the same terms as Perl itself.
 #
@@ -15,7 +15,7 @@
 package Tk::Date;
 use Time::Local qw(timelocal);
 use strict;
-use vars qw($VERSION @ISA $has_numentryplain $has_numentry
+use vars qw($VERSION @ISA $DEBUG $has_numentryplain $has_numentry
 	    @monlen %choice $en_weekdays $en_monthnames
 	    $weekdays $monthnames
 	   );
@@ -23,7 +23,8 @@ use Tk::Frame;
 @ISA = qw(Tk::Frame);
 Construct Tk::Widget 'Date';
 
-$VERSION = '0.42';
+$VERSION = '0.43';
+$VERSION = eval $VERSION;
 
 @monlen = (undef, 31, undef, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
  # XXX DST?
@@ -585,6 +586,10 @@ sub state {
 	    eval '$ww->configure("-state" => $state);';
 	    #warn "$ww: $@" if $@;
 	}
+	my $chooser = $w->Subwidget("chooser");
+	if (Tk::Exists($chooser)) {
+	    $chooser->configure(-state => $state);
+	}
 	$w->{Configure}{"-state"} = $state;
     } else {
 	$w->{Configure}{"-state"};
@@ -824,7 +829,7 @@ sub set_date {
 	if ($key eq 'A') {
 	    $sw->configure(-text => $value);
 	} else {
-	    my $v = sprintf("%0".$w->{'len'}{$key}."d", $value);
+	    my $v = sprintf("%0".($w->{'len'}{$key}||"")."d", $value);
 	    if ($sw->isa('Tk::Entry') ||
 		$sw->isa('Tk::NumEntry')) {
 		$sw->delete(0, 'end');
@@ -871,13 +876,22 @@ sub _monlen {
 
 sub _get_week_days {
     return $weekdays if $weekdays;
+    eval {
+	my $loc = _get_datetime_locale();
+	my @weekdays = @{ $loc->day_names }; # clone!
+	unshift @weekdays, pop @weekdays;
+	$weekdays = \@weekdays;
+    };
+    return $weekdays if $weekdays;
+    warn $@ if $@ && $DEBUG;
     eval {
 	require POSIX;
 	POSIX->VERSION(1.03);
 	# prefer POSIX because of localized weekday names
+	my $locale_charset = _guess_time_locale_charset();
 	my $_weekdays = [];
 	foreach my $day_i (6 .. 12) { # 2000-08-06 till 2000-08-12
-	    my $wday = POSIX::strftime("%A", 0,0,0,$day_i,8-1,2000-1900);
+	    my $wday = _decoded_strftime("%A", [0,0,0,$day_i,8-1,2000-1900], $locale_charset);
 	    if ($wday eq '' || $wday =~ /^\?/) {
 		die "Can't get weekday name from locale";
 	    }
@@ -885,7 +899,7 @@ sub _get_week_days {
 	}
 	$weekdays = $_weekdays;
     };
-    warn $@ if $@;
+    warn $@ if $@ && $DEBUG;
     if (!$weekdays) {
 	$weekdays = $en_weekdays;
     }
@@ -894,12 +908,20 @@ sub _get_week_days {
 
 sub _get_month_names {
     return $monthnames if $monthnames;
+    eval {
+	my $loc = _get_datetime_locale();
+	my @monthnames = @{ $loc->month_names }; # clone!
+	$monthnames = \@monthnames;
+    };
+    return $monthnames if $monthnames;
+    warn $@ if $@ && $DEBUG;
     eval {
 	require POSIX;
 	# prefer POSIX because of localized month names
+	my $locale_charset = _guess_time_locale_charset();
 	my $_monthnames = [];
 	foreach my $month_i (1 .. 12) {
-	    my $mname = POSIX::strftime("%B", 0,0,0,1,$month_i-1,1970);
+	    my $mname = _decoded_strftime("%B", [0,0,0,1,$month_i-1,1970], $locale_charset);
 	    if ($mname eq '' || $mname =~ /^\?/) {
 		die "Can't get month name from locale";
 	    }
@@ -1013,6 +1035,80 @@ sub _Destroyed {
     $w->SUPER::DESTROY($w);
 }
 
+# Call only if POSIX is already loaded
+sub _guess_time_locale_charset {
+    my $charset;
+    my $locale_name = eval {
+	# Is setlocale and LC_TIME available everywhere?
+	POSIX::setlocale(POSIX::LC_TIME());
+    };
+    warn $@ if $@ && $DEBUG;
+    my $full_locale_name = $locale_name;
+    $locale_name =~ s{^[^.]+\.}{};
+    $locale_name =~ s{\@.*}{};
+    if ($locale_name) {
+	if      ($locale_name =~ m{^utf-?8$}i) {
+	    $charset = "utf-8";
+	} elsif ($locale_name =~ m{^iso[-_]?8859-?(\d+)$}i) {
+	    $charset = "iso-8859-$1";
+	} elsif ($locale_name =~ m{^(?:cp|ansi)-?(\d+)$}i) {
+	    $charset = "cp$1";
+	} elsif ($locale_name =~ m{^koi8-.$}i) {
+	    $charset = lc $locale_name;
+	} elsif ($locale_name =~ m{^euc-?(cn|jp|kr)$}i) {
+	    $charset = "euc-" . lc $1;
+	} elsif ($locale_name =~ m{^euc$}i && $full_locale_name =~ m{(kr|cn|jp)}i) {
+	    $charset = "euc-" . lc $1;
+	} elsif ($locale_name =~ m{^gb-?18030$}i && eval { require Encode::HanExtra; 1; }) {
+	    $charset = "gb18030";
+	} elsif ($locale_name =~ m{^gb-?(\d+|k)$}i) {
+	    $charset = "gb" . lc $1;
+	} elsif ($locale_name =~ m{^big5-?hkscs$}i) {
+	    $charset = "big5-hkscs";
+	} elsif ($locale_name =~ m{^big5$}i) {
+	    $charset = "big5";
+	} elsif ($locale_name =~ m{^s(?:hift)?-?jis$}i) {
+	    $charset = "shiftjis";
+	} elsif ($locale_name =~ m{^(?:us-)?ascii$}i) {
+	    $charset = "ascii";
+	}
+	## More encodings are missing:
+	## (Following seen on FreeBSD 6.2)
+	##   cp1131: http://source.icu-project.org/repos/icu/icu/trunk/source/data/mappings/ibm-1131_P100-1997.ucm
+	##   iscii-dev
+	##   armscii-8
+	##   pt154 (kazachstan?)
+	## (Following seen on Solaris 10)
+	##   PCK (japanese)
+	##   TIS620 (thai)
+	##   BIG5HK
+	##   zh_TW.EUC (use euc-cn?)
+    }
+    $charset;
+}
+
+# Call only if POSIX is already loaded
+sub _decoded_strftime {
+    my($fmt, $localtime_ref, $locale_charset) = @_;
+    my $date_string = POSIX::strftime($fmt, @$localtime_ref);
+    return $date_string if (!$locale_charset);
+    eval {
+	require Encode;
+	$date_string = Encode::decode($locale_charset, $date_string, Encode::LEAVE_SRC());
+    };
+    warn $@ if $@ && $DEBUG;
+    return $date_string;
+   
+}
+
+sub _get_datetime_locale {
+    require DateTime::Locale;
+    require POSIX;
+    my $locale_name = POSIX::setlocale(POSIX::LC_TIME());
+    my $loc = DateTime::Locale->load($locale_name);
+    $loc;
+}
+
 ######################################################################
 
 package Tk::Date::UnixTime;
@@ -1147,10 +1243,10 @@ the second method as B<datehash>.
 
 =head1 STANDARD OPTIONS
 
-=over 4
-
 Tk::Date descends from Frame and inherits all of its options.
 
+=over 4
+
 =item -orient
 
 Specified orientation of the increment and decrements buttons. May be
@@ -1169,8 +1265,8 @@ Tk-GBARR distribution are installed too.
 
 If true then all entry fields will obtain arrows. Otherwise only one
 arrow pair for each date and time will be drawn. This option can be
-set only while creating the widget. This option needs the Tk::NumEntry
-widget to be installed.
+set only while creating the widget. This option needs the
+L<Tk::NumEntry> widget to be installed.
 
 =item -bell
 
@@ -1278,10 +1374,10 @@ command will not be executed.
 
 =item -readonly
 
-If set to a true value, then it is only possible to change the values
-by pressing the increment/decrement buttons. This option is only
-useful if C<-allarrows> is also set and if the user has
-C<Tk::NumEntry> installed on his system.
+"readonly" means only that the entry fields are readonly. However, the
+user is still able to use the increment/decrement buttons to change
+the date value. Use C<< -state => "disabled" >> to make a date widget
+completely unchangable by the user.
 
 =item -repeatinterval
 
@@ -1418,8 +1514,8 @@ Slaven Rezic <eserte@cs.tu-berlin.de>
 
 =head1 COPYRIGHT
 
-Copyright (c) 1997, 1998, 1999, 2000 Slaven Rezic. All rights
-reserved. This module is free software; you can redistribute it and/or
-modify it under the same terms as Perl itself.
+Copyright (C) 1997, 1998, 1999, 2000, 2001, 2005, 2007 Slaven Rezic.
+All rights reserved. This module is free software; you can
+redistribute it and/or modify it under the same terms as Perl itself.
 
 =cut
